"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.getLocal = getLocal;
exports.initialize = initialize;
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/for-each"));
var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/filter"));
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/keys"));
var _defineProperties = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/object/define-properties"));
/* eslint-disable no-underscore-dangle,no-use-before-define */
/**
 * This is a factory that creates a css modules that are:
 * 1. compatible with webpack's extract-text-webpack-plugin
 * 2. compatible with webpack's style-loader
 * 3. maintains the same interface when used outside of webpack.
 *
 * Because I tried to aim for these 3 things, there are a few questionable decisions that I've made
 * (from the perspective of writing clean code).
 *
 * Gist:
 *   The resulting module is an array of arrays, (the null can probably be removed, was just added because
 *   this is how webpack-contrib/css-loader represents css-modules):
 *
 *      [ [module-id, css, null] ]
 *
 *  This array also has all of the variables defined in the css (be it, unique class mappings or
 *  @value definitions) attached at the top level. This means that everything is in the same level.
 *
 *      var module = builder.initialize(/* ommited *\/)
 *      module[0] // => entrypoint module (represented as an array)
 *      module.localVar // => local value
 *      module.locals.localVar // => local value
 *
 *  This is not ideal because it makes for a really messy module.
 *
 */

const {
  stringify
} = JSON;
const {
  isArray,
  prototype: {
    forEach,
    reduce,
    push
  }
} = Array;
const isEnumerable = Object.prototype.propertyIsEnumerable;
function getId(module) {
  return module[0][0];
}
/**
 * Ensures it is an enumerable property that is not an array element
 */
function isALocalDefinition(local) {
  return !/^\d+$/.test(local);
}
function createLocals() {
  var _context, _context2;
  const locals = {};
  (0, _forEach.default)(_context = (0, _filter.default)(_context2 = (0, _keys.default)(this)).call(_context2, isALocalDefinition)).call(_context, local => {
    locals[local] = this[local];
  });
  return locals;
}

/* Shared counter to guarantee unique ids for nonCSSModule imports */
let nonCSSModuleImportId = 0;
function normalizeImport(parentModule, requiredModule) {
  if (requiredModule.__is_css_module__) {
    return requiredModule;
  }
  if (typeof requiredModule.toCssString === 'function') {
    /* eslint-disable no-plusplus */
    const id = `${getId(parentModule)}/import__${nonCSSModuleImportId++}`;
    const css = requiredModule.toCssString();
    return [[id, css, null]];
  }
  return [];
}
function eachLocalValue(module, localName, processValue) {
  var _context3;
  (0, _forEach.default)(_context3 = getLocal(module, localName).split(/\s+/)).call(_context3, processValue);
}
function composeLocals(localValues) {
  const uniqueValues = {};
  forEach.call(localValues, valueOrImport => {
    /* Its an import if its an array */
    if (isArray(valueOrImport)) {
      eachLocalValue(valueOrImport[0], valueOrImport[1], importedValue => {
        uniqueValues[importedValue] = true;
      });
      return;
    }
    uniqueValues[valueOrImport] = true;
  });
  return (0, _keys.default)(uniqueValues).join(' ');
}
function eachLocalKV(localDefinitions, processLocalDefinition, thisArg) {
  forEach.call(localDefinitions, _ref => {
    let [keys, values] = _ref;
    forEach.call(keys, key => {
      processLocalDefinition.call(thisArg, key, composeLocals(values));
    });
  });
}
function toString() {
  return reduce.call(this, (cssString, subModule) => `${subModule[1]}${cssString}`, '');
}
function importModule(cssModule) {
  forEach.call(normalizeImport(this, cssModule), subModule => {
    const id = subModule[0];
    if (!this.__imported_modules__[id]) {
      this.__imported_modules__[id] = true;
      push.call(this, subModule);
    }
  });
}
function importEach(cssModuleList) {
  forEach.call(cssModuleList, importModule, this);
}
function defineLocals(localDefinitions) {
  eachLocalKV(localDefinitions, (key, value) => {
    this[key] = value;
  });
}
function strictGetLocal(localName) {
  const local = getLocal(this, localName);
  if (local) {
    return local;
  }
  throw new Error(`${stringify(localName)} is not defined in ${stringify(this.locals, null, 2)}`);
}
const CSSModuleStaticProperties = {
  __is_css_module__: {
    value: true
  },
  defineLocals: {
    value: defineLocals
  },
  get: {
    value: strictGetLocal
  },
  importEach: {
    value: importEach
  },
  locals: {
    get: createLocals
  },
  toString: {
    value: toString
  }
};
function initialize(moduleId, css) {
  const module = [[moduleId, css, null]];
  (0, _defineProperties.default)(module, CSSModuleStaticProperties);
  (0, _defineProperties.default)(module, {
    __imported_modules__: {
      value: {}
    }
  });
  return module;
}
function getLocal(module, localName) {
  if (typeof module.locals === 'object') {
    return module.locals[localName] || '';
  }
  const isValidLocalRequest = isEnumerable.call(module, localName) && isALocalDefinition(localName);
  if (module[localName] && isValidLocalRequest) {
    return module[localName];
  }
  return '';
}